const request = require("request"),
  http = require('http'),
  cmd = require('../libs/cmd');

const get = (url) => {
  return new Promise((resolve, reject) => {
    http.get(url, (res) => {
      const {statusCode} = res;
      const contentType = res.headers['content-type'];

      let error;
      if (statusCode !== 200) {
        error = new Error('Request Failed.\n' +
          `Status Code: ${statusCode}`);
      } else if (!/^application\/json/.test(contentType)) {
        error = new Error('Invalid content-type.\n' +
          `Expected application/json but received ${contentType}`);
      }
      if (error) {
        reject(error.message);
        // consume response data.list to free up memory
        res.resume();
        return;
      }

      res.setEncoding('utf8');
      let rawData = '';
      res.on('data', (chunk) => {
        rawData += chunk;
      });
      res.on('end', () => {
        try {
          const parsedData = JSON.parse(rawData);
          resolve(parsedData);
        } catch (e) {
          reject(e.message);
        }
      });
    }).on('error', (e) => {
      reject(`Got error: ${e.message}`);
    });
  });
};
const getCert = (bundle_id, api_token) => {
  let options = {
    method: 'POST',
    url: 'http://api.fir.im/apps',
    headers: {
      'Cache-Control': 'no-cache',
      'Content-Type': 'application/json'
    },
    formData: {
      type: 'android',
      bundle_id,
      api_token
    }
  };
  return new Promise((resolve, reject) => {
    request(options, function (error, response, body) {
      if (error) reject(error);

      try {
        let data = JSON.parse(body);
        resolve(data.cert.binary);
      } catch (e) {
        reject(e);
      }
    });
  })
};
const getFirInfo = (packageName, api_token) => {
  return get(`http://api.fir.im/apps/latest/${packageName}?type=android&api_token=${api_token}`);
};
const upload = ({api_token, file, apkName, versionName, packageName, versionCode, apkEName}, updatelog) => {
  return new Promise((resolve, reject) => {
    getCert(packageName, api_token).then(({key, token}) => {
      let cd = `firuploadapk ${key} ${token} ${file} ${apkName} ${versionName} ${versionCode} ${updatelog}`;
      console.log(cd)
      return cmd(cd);
    }).then(() => {
      return getFirInfo(packageName, api_token);
    }).then(({name, build, update_url, versionShort}) => {
      if (build === versionCode + "" && versionShort === versionName) {
        resolve({
          isok: true,
          info: {
            api_token,
            file,
            apkName,
            apkEName,
            versionName,
            packageName,
            versionCode,
            fir: {name, build, update_url, versionShort}
          }
        });
      } else {
        resolve({
          isok: false,
          info: {
            api_token,
            file,
            apkName,
            apkEName,
            versionName,
            packageName,
            versionCode,
            fir: {name, build, update_url, versionShort},
            err: "build 或者 versionShort不匹配"
          }
        });
      }
    }).catch(e => {
      resolve({
        isok: false,
        info: {
          api_token,
          file,
          apkName,
          apkEName,
          versionName,
          packageName,
          versionCode,
          fir: {},
          err: e.message + "\n" + e.stack
        }
      });
    });
  })
};

module.exports = {upload, getFirInfo};

