/**
 * http获取数据并选择存储
 */

var http = require('http');
var https = require('https');
var fs = require('fs');
var config = require('../../config');


var androidPath = config.oemParms[config.type].androidPath;

function rClient(url = "") {
  switch (url.split(':')[0]) {
    case 'http':
      return http;
    case 'https':
      return https;
    default:
      throw new Error("download url must be http/https: \nurl=" + url);
  }
}

exports.get = (url) => {
  return new Promise((resolve, reject) => {
    rClient(url).get(url, function (res) {
      console.log("Got response: " + res.statusCode);
      if (res.statusCode !== 200) {
        reject('Request Failed.\n Status Code: ${res.statusCode}');
        return
      }

      res.setEncoding('utf8');
      var rawData = '';
      res.on('data', (chunk) => rawData += chunk);
      res.on('end', () => {
        try {
          var parsedData = JSON.parse(rawData);
          resolve(parsedData);
        } catch (e) {
          console.log(e.message);
          reject('Json:' + e.message);
        }
      });
    }).on('error', function (e) {
      reject('http:' + e.message);
    });
  });

}


/**
 * 下载批量图片
 * @param data_arr [{logo144:'',oem:''},..]
 * @returns {Promise}
 */
exports.downImagesPromise = (filePath, data_arr) => {
  return new Promise((resolve, reject) => {
    _downImages(filePath, data_arr, resolve, reject)
  });
}

function _downImages(filePath, arr, resolve, reject) {
  var map = arr[0];
  if (map) {
    rClient(map.logo144).get(map.logo144, function (res) {
      var imgData = "";
      res.setEncoding("binary");
      res.on("data", function (chunk) {
        imgData += chunk;
      });

      res.on("end", function () {
        var fileDir = filePath + "/ic_" + map.oem + ".png";
        fs.writeFile(fileDir, imgData, "binary", function (err) {
          if (err) {
            console.log(fileDir + " down fail:" + err);
            reject(fileDir + " down fail:" + err);
          } else {
            console.log(" down success :" + fileDir);
            arr.splice(0, 1);
            _downImages(filePath, arr, resolve, reject);
          }
        });
      });
    }).on('error', function (e) {
      reject("http-img error: " + e.message);
    });
    ;
  } else {
    resolve();
  }
}

/**
 * 下载文件
 * @param file
 * @param fileUrl
 */
exports.downloadFile = (file, fileUrl) => {
  return new Promise((resolve, reject) => {
    if (fileUrl) {
      rClient(fileUrl).get(fileUrl, function (res) {
        var fileData = "";
        res.setEncoding("binary");
        res.on("data", function (chunk) {
          fileData += chunk;
        });

        res.on("end", function () {
          fs.writeFile(file, fileData, "binary", function (err) {
            if (err) {
              console.log(file + " down fail:" + err);
              reject(file + " down fail:" + err);
            } else {
              console.log(file + " down success");
              resolve();
            }
          });
        });
      }).on('error', function (e) {
        reject("http-file error: " + e.message);
      });
      ;
    }
  });
};

/**
 * 上传apk
 * @param fullname
 */
var uploadApks = function (fullnames) {
  //TODO 缺少身份认证  防止其他人上传功能
  //现在计划将文件防止项目外，即只有我这台电脑才能完成上传功能
  var fullname = fullnames[0];
  if (fullname) {
    console.log(fullname + ' upload start.');
    var boundaryKey = '----' + new Date().getTime();
    var options = {
      host: config.uploadInfo.host,//远端服务器域名
      port: config.uploadInfo.port,//远端服务器端口号
      method: 'POST',
      path: config.uploadInfo.path,//上传服务路径
      headers: {
        'Content-Type': 'multipart/form-data; boundary=' + boundaryKey,
        'Connection': 'keep-alive'
      }
    };
    var req = http.request(options, function (res) {
      res.setEncoding('utf8');
      res.on('data', function (chunk) {
        //console.log('body: ' + chunk);
      });
      res.on('end', function () {
        console.log(fullname + ' upload end.');
        fullnames.splice(0, 1);
        uploadApks(fullnames);
      });

      req.on('error', function (e) {
        console.error("error:" + e);
        fullnames.splice(0, 1);
        uploadApks(fullnames);
      });
    });
    req.write(
      '--' + boundaryKey + '\r\n' +
      'Content-Disposition: form-data; name="upload"; filename="' + fullname + '"\r\n' +
      'Content-Type: application/vnd.android.package-archive\r\n\r\n'
    );
    //设置1M的缓冲区
    var fileStream = fs.createReadStream(androidPath + config.androidApkPath + fullname, {bufferSize: 1024 * 1024});
    fileStream.pipe(req, {end: false});
    fileStream.on('end', function () {
      req.end('\r\n--' + boundaryKey + '--');
    });
  }
}

exports.uploadApks = uploadApks;



