#!/usr/bin/env node
var packageInfo = require('../package.json'),
  config = require('../config'),
  proConfig = require('../config/product.json'),
  commander = require('commander'),
  jsonfile = require('jsonfile'),
  packing = require('../js/packing/index'),
  file = __dirname + '/../config/product.json',
  fs = require('fs'),
  fir = require('../js/fir'),
  apkparse = require('../js/apkparse'),
  {updateAndroid} = require('../js/oss');


commander
  .version(packageInfo.version)
  .description("this is a appliction for packing android apk");

commander
  .command("show [keys...]")
  .description('show the settings')
  .action((keys) => {
    showSettings(keys);
  });

commander
  .command("set <key> <value>")
  .description('基础设置配置; keys:[type,system,oemFil,androidImagPath,androidApkPath]')
  .action((k, v) => {
    switch (k) {
      case 'system':
      case 'oemUrl':
        setProduct(k, v);
    }
  });

commander
  .command("type <type>")
  .description('类型选择，type与oemParms中的key对应')
  .action((type) => {
    var b = false;
    Object.keys(config.oemParms).forEach((key) => {
      if (key === type) {
        b = true;
        return setProduct('type', key);
      }
    });
    !b && console.log('请选择"%s"中的一个', JSON.stringify(Object.keys(config.oemParms)))
  });

commander
  .command("setParms <type> [androidPath] [oemUrl]")
  .description('设置oemParmss中的值，即添加androidPath,oemUrl配置, 不设置值表示删除该项')
  .action((type, androidPath, oemUrl) => {
    if (androidPath && oemUrl) {
      setParms(type, {androidPath, oemUrl});
    } else if ((!androidPath) && (!oemUrl)) {
      if (type === 'app') return console.log('pos为必有项，不能删除');
      deleteParms(type);
    } else {
      console.log('请输入正确的参数');
    }
  });

commander
  .command("list")
  .description('获得oem的列表')
  .action(() => {
    packing.oemList();
  });

commander
  .command("packing [packings...]")
  .description('android 打包，没参数返回list，all表示全部打包')
  .action((packings) => {
    var _l = () => {
      console.log('请输入packings,packings如下：');
      packing.oemList();
    };

    packings && packings.length > 0 ? packing.packingApk(packings[0] === 'all' ? ['*'] : packings) : _l();
  });

commander
  .command("test")
  .description('请使用"oemTest <paths> <ext>"命令')
  .action(() => {
    console.log('请使用"oemTest <paths> <ext>"命令');
  });

commander
  .command("firupload <packingFilePath>")
  .description('上传文件到fir，需要oem packing生产的packing.list文件路径作为参数传入')
  .action(filePath => {
    fir.uploads(filePath);
  });

commander
  .command("apkparse <apks...>")
  .description('查看Android包信息')
  .action(apks => {
    apkparse.analysisApks(apks, true);
  });

commander
  .command("openUpdate <apk> <appDescription>")
  .description('打开caibao或caibaolm更新')
  .action((apk, appDescription) => {
    appDescription = appDescription.replace(/\\n/g, "\n");
    updateAndroid(apk, appDescription);
  });

commander.parse(process.argv);

function showSettings(keys) {
  if (keys && keys.length > 0) {
    keys.forEach((d) => {
      console.log('"%s"->"%s"', d, JSON.stringify(config[d], null, 2));
    })
  } else {
    Object.keys(config).forEach(function (key) {
      console.log('%s->%s', key, JSON.stringify(config[key], null, 2));
    });
  }
}

function setProduct(k, v) {
  var p = {};
  p[k] = v;
  jsonfile.writeFile(file, Object.assign(proConfig, p), function (err) {
    if (err) {
      console.error('err:' + err)
    }
  });
}

function deleteParms(k) {
  if (proConfig.oemParms) {
    delete proConfig.oemParms[k];
    jsonfile.writeFile(file, proConfig, function (err) {
      if (err) {
        console.error('err:' + err)
      }
    });
  }
}

function setParms(k, v) {
  if (!proConfig.oemParms) {
    proConfig.oemParms = config.oemParms;

  }
  proConfig.oemParms[k] = v;

  jsonfile.writeFile(file, proConfig, function (err) {
    if (err) {
      console.error('err:' + err)
    }
  });
}
